@@ -1,6 +1,3 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 from mpl_toolkits.axisartist.grid_finder import (
     FormatterPrettyPrint,
     MaxNLocator)
